/* hdlcrx.c v1.0 2016/11/21
 * Program which receives HDLC Frames on STDIN and prints the binary Payload on STDOUT
 * if a jitter of more than 5ms occurs it prints a message on STDERR
 *
 * (C) 2017 Copyright beroNet GmbH
 * Author: Christian Richter
 * Contact: info@beronet.com
 *
 * This program is licensed by beroNet and may only be used in the PoC for the project: 
 * 'IP22' - Media Gateway ( or higher IPxx Versions)
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>

#define FAST_HDLC_NEED_TABLES
#include "fasthdlc.h"

#include <sys/resource.h>

int which = PRIO_PROCESS;
pid_t pid;
int priority = -20;
int ret;

int main(int argc, char *argv[])
{
	unsigned char decbuf[1024];
	unsigned char actual[1024];
	int res;
	int datain;
	int hdlcin;
	int hdlccnt;
	int x;
	struct fasthdlc_state receiver;
	char buf[161];
	struct timeval tv_store, tv;
	int jitter_cnt = 0;

	pid = getpid();
	ret = setpriority(which, pid, priority);
	
	fasthdlc_precalc();
	
	fasthdlc_init(&receiver, FASTHDLC_MODE_64);
	
	hdlccnt = 0;

	gettimeofday(&tv_store, NULL);
	for (;;) {
		/* Feed in some input */
		int i;
		int l=read(0, buf, sizeof(buf));

		if (l<=0) break;
		
		for (i=0; i<l; i++) {
			if (fasthdlc_rx_load(&receiver, buf[i])) {
				fprintf(stderr, "Unable to feed receiver :(\n");
				exit(1);
			}
			res = fasthdlc_rx_run(&receiver);
			if (res & RETURN_EMPTY_FLAG)
				continue;
			if (res & RETURN_COMPLETE_FLAG) {
				if (hdlccnt) {
					int r;
					if (jitter_cnt == 10) {
						time_t delta_s;
						suseconds_t delta_u;
						long delta;

						gettimeofday(&tv, NULL);
						delta_s = tv.tv_sec - tv_store.tv_sec;
						delta_u = tv.tv_usec - tv_store.tv_usec;
						delta = delta_s*1000000 + delta_u - 20000*jitter_cnt;

						if (delta/jitter_cnt > 5000 ) fprintf(stderr, "%s 5 ms jitter detected: %ld us\n", argv[1], delta);

						memcpy(&tv_store, &tv, sizeof(tv));
						jitter_cnt = 0;
					}

					r=write(1, decbuf, hdlccnt-2); //skip CRC
					if (r<0) break;
					/* Reset message receiver */
					hdlccnt = 0;
					jitter_cnt ++;
				}
			} else if (res & RETURN_DISCARD_FLAG) {
				if (1 || hdlccnt) {
					fprintf(stderr, "Detected abort :(\n");
					exit(1);
				}
			} else {
				decbuf[hdlccnt++] = res;
			}
		}
	}
}
